//+------------------------------------------------------------------+
//|                                        PivotsTrailStops_v1.5.mq4 |
//|                           Copyright  2009, TrendLaboratory Ltd. |
//|            http://finance.groups.yahoo.com/group/TrendLaboratory |
//|                                   E-mail: igorad2003@yahoo.co.uk |
//+------------------------------------------------------------------+
#property copyright "Copyright  2009, TrendLaboratory Ltd."
#property link      "http://finance.groups.yahoo.com/group/TrendLaboratory"

#include <stdlib.mqh>

//---- input parameters
extern string     Expert_Name    = " PivotsTrailStops_v1.5";

extern int        Magic          =     0;    // Magic Number
extern int        Slippage       =     3;
extern int        TrailStopMode  =     1;    // 0-off,1-on
extern int        StopLossLevel  =     1;    // 0-off,1-1st nearest Pivot,2-2nd nearest and so on
extern int        MinStopLoss    =    10;    // Min StopLoss in pips 
extern double     TargetPct1     =    50;    // % of full Order Lots for 1st Target Close(0-off)
extern double     TargetPct2     =    30;    // % of full Order Lots for 2nd Target Close(0-off)
extern double     TargetPct3     =    20;    // % of full Order Lots for 3rd Target Close(0-off)
extern int        MinProfit      =     5;    // Min Profit in pips 
extern int        DebugMode      =     0;    // Switch of Debug mode (0-off;1-on) 

extern string     PivotInputs    = " Pivots inputs";
extern int        DailyPivotHour =     1;    // Daily Pivot Hour (ex. New York Close Time...)  
extern int        DaysNumber1    =     5;    // Days Number for 5 Days Average Range
extern int        DaysNumber2    =    10;    // Days Number for 10 Days Average Range
extern int        DaysNumber3    =    20;    // Days Number for 20 Days Average Range

//-----
int      TriesNum=5;
double   range[];
datetime ny_time[];
int      ny_shift[];
datetime pTime, prevTime, pDay;
double   _Point;
bool     fTime = true;
double   Target[1,10];
double   lot_step;
double   minstop;
double   aPivot[21];
//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
{
//---- 
   pTime    = iTime(Symbol(),0,0);
   _Point   = MarketInfo(Symbol(),MODE_POINT)*MathPow(10,Digits%2);
   minstop  = MarketInfo(Symbol(),MODE_STOPLEVEL);
   lot_step = MarketInfo(Symbol(),MODE_LOTSTEP); 
//----
   return(0);
}

// ---- Scan Trades
int ScanTrades(int mode)
{   
   int total = OrdersTotal();
   int numords = 0;
   bool type = false; 
   
   
   for(int cnt=0; cnt<total; cnt++) 
   {        
   OrderSelect(cnt, SELECT_BY_POS);            
   if (mode == 0) type = OrderType()<=OP_SELLSTOP;
   if (mode == 1) type = OrderType()<=OP_SELL;   
   if (mode == 2) type = OrderType()>OP_SELL && OrderType()<=OP_SELLSTOP; 
   
   if(OrderSymbol() == Symbol() && type && OrderMagicNumber() == Magic)  
   numords++;
   }
   return(numords);
}  


//---- Calculation of Pivots 
double Pivot(int num)
{
   int k;
   double pivot = 0;
   
   ArrayResize(ny_time,DaysNumber3+1);
   ArrayResize(ny_shift,DaysNumber3+1);
   ArrayResize(range,DaysNumber3);   
   
   for (int shift=0;shift<=DaysNumber3;shift++)
   {
   ny_time[shift]  = iTime(NULL,PERIOD_D1,shift) + (DailyPivotHour-Period()/60.0)*3600;
   ny_shift[shift] = iBarShift(NULL,0,ny_time[shift]);
   }
   
   for (shift=0;shift<=DaysNumber3-1;shift++)
   {
   int length = ny_shift[shift+1] - ny_shift[shift];
   double high  = High[iHighest(NULL,0,MODE_HIGH,length,ny_shift[shift])];
   double low   = Low[iLowest(NULL,0,MODE_LOW,length,ny_shift[shift])];
   range[shift] = high - low; 
   //Print("L=",length," R=", range[shift]," h=",high," l=",low," ny_t=",TimeToStr(ny_time[shift])," shi=",ny_shift[shift]);
   }
   
   double aR1   = 0; 
   double R5    = 0;
   double R10   = 0;
   double R20   = 0;
   double Ravg  = 0;  
   double sumR5 = 0; 
   double sumR10 = 0;
   double sumR20 = 0;
   
   for(k=0;k<DaysNumber1;k++)   sumR5    += range[k];
   for(k=0;k<DaysNumber2;k++)   sumR10   += range[k];
   for(k=0;k<DaysNumber3;k++)   sumR20   += range[k];

   aR1   = NormalizeDouble(range[0],Digits); 
   R5    = NormalizeDouble(sumR5/DaysNumber1,Digits);
   R10   = NormalizeDouble(sumR10/DaysNumber2,Digits);
   R20   = NormalizeDouble(sumR20/DaysNumber3,Digits);
   Ravg  = NormalizeDouble((aR1+R5+R10+R20)/4,Digits);  
    
   double P  = iClose(NULL,0,ny_shift[0]);
   //Print("ny_t=",TimeToStr(ny_time[0])," P=", P," Ravg=",Ravg);
   
   pivot = NormalizeDouble(P + 0.25 * num * Ravg, Digits);
   
   return(pivot);
}   

double TotalProfit(int type)
{
   int total=OrdersTotal();
   double totalPips = 0;
   double totalProfits = 0;
   for (int cnt=0;cnt<total;cnt++)
   { 
   OrderSelect(cnt, SELECT_BY_POS);   
   int mode=OrderType();
   bool condition = false;
   if(OrderSymbol() == Symbol())
      if(Magic>0 && OrderMagicNumber() == Magic ) condition = true;
      else 
      if(Magic == 0) condition = true;   
      
      if (condition)
      {      
         switch (mode)
         {
         case OP_BUY:
            totalPips += (MarketInfo(OrderSymbol(),MODE_BID)-OrderOpenPrice())/_Point;
            totalProfits += OrderProfit() + OrderSwap() + OrderCommission();
            break;
            
         case OP_SELL:
            totalPips += (OrderOpenPrice()-MarketInfo(OrderSymbol(),MODE_ASK))/_Point;
            totalProfits += OrderProfit() + OrderSwap() + OrderCommission();
            break;
         }
      }            
	}
   if (type == 1) double result = totalPips;
   else result = totalProfits;
   
   return(result);
}

void ChartComment()
{
   string sComment   = "";
   string sp         = "----------------------------------------\n";
   string NL         = "\n";
       
   sComment = sp;
   sComment = sComment+"ExpertName : "+Expert_Name+NL;
   sComment = sComment+"Open Orders= "+ScanTrades(1)+NL;
   sComment = sComment+"Current Profit(pips)= " + DoubleToStr(TotalProfit(1),2) + NL;
   sComment = sComment+"Current Profit("+ AccountCurrency()+") = " + DoubleToStr(TotalProfit(0),2) + NL; 
   sComment = sComment+sp;
   sComment = sComment+"Pivots: P="+DoubleToStr(aPivot[10],Digits)+NL;
   sComment = sComment+"R1="+DoubleToStr(aPivot[11],Digits)+" R2="+DoubleToStr(aPivot[12],Digits)+" R3="+DoubleToStr(aPivot[13],Digits)+" R4="+DoubleToStr(aPivot[14],Digits)+NL;
   sComment = sComment+"S1="+DoubleToStr(aPivot[9],Digits)+" S2="+DoubleToStr(aPivot[8],Digits)+" S3="+DoubleToStr(aPivot[7],Digits)+" S4="+DoubleToStr(aPivot[6],Digits)+NL;
   
   Comment(sComment);
}

void TrailStop()
{
   int    error;  
   bool   result=false;
   double BuyStop=0, SellStop=0;
    
   for (int cnt=0;cnt<OrdersTotal();cnt++)
   { 
   OrderSelect(cnt, SELECT_BY_POS);   
   int mode=OrderType();    
      if (OrderSymbol() == Symbol() && OrderMagicNumber() == Magic) 
      {
         if (mode==OP_BUY) 
         {
			   int i=0;
			   while(i <= 20)
			   { 
			   if (aPivot[i] >= OrderOpenPrice() && Low[1] > aPivot[i]) BuyStop = aPivot[i]; 
			   i++;
			   }			   
			   if(BuyStop > 0 && Bid - BuyStop < minstop * Point) BuyStop = Bid - minstop * Point;
			   //if(DebugMode > 0) Print("BuyStop=",BuyStop);
			   if(NormalizeDouble(OrderOpenPrice(),Digits) <= BuyStop) 
            {   
			      if (BuyStop > NormalizeDouble(OrderStopLoss(),Digits) || OrderStopLoss() == 0) 
			      {
			         for(int k = 0 ; k < TriesNum; k++)
                  {
                  result = OrderModify(OrderTicket(),OrderOpenPrice(),
			                              BuyStop,
			                              OrderTakeProfit(),0,Lime);
                  error=GetLastError();
                     if(error==0) break;
                     else {Sleep(5000); RefreshRates(); continue;}
                  }   		 
               }            
            }
         }   
// - SELL Orders          
         if (mode==OP_SELL)
         {
            i = 20;
			   while(i >= 0)
			   { 
			   if (aPivot[i] <= OrderOpenPrice() && High[1] < aPivot[i]) SellStop = aPivot[i];  
			   i--;
			   }		 
            if(SellStop > 0 && SellStop - Ask < minstop * Point) SellStop = Ask + minstop * Point;
            //if(DebugMode > 0) Print("SellStop=",SellStop);
            if(NormalizeDouble(OrderOpenPrice(),Digits) >= SellStop && SellStop > 0) 
            {
               if(SellStop < NormalizeDouble(OrderStopLoss(),Digits) || OrderStopLoss() == 0) 
               {
                  for( k = 0 ; k < TriesNum; k++)
                  {
                  result = OrderModify(OrderTicket(),OrderOpenPrice(),
			                              SellStop,
			                              OrderTakeProfit(),0,Orange);
                  error=GetLastError();
                     if(error==0) break;
                     else {Sleep(5000); RefreshRates(); continue;}
                  }
               }   
   			}	    
         }
      }
   }     
}   

void InitialStop()
{
   int    error, np;  
   bool   result=false;
   double BuyStop=0, SellStop=0;
    
   for (int cnt=0;cnt<OrdersTotal();cnt++)
   { 
   OrderSelect(cnt, SELECT_BY_POS);   
   int mode=OrderType();    
      if (OrderSymbol() == Symbol() && OrderMagicNumber() == Magic && OrderStopLoss() == 0) 
      {
         if (mode==OP_BUY) 
         {
			   int i = 20;
			   while(i >= 0)
			   { 
			   if (aPivot[i] < OrderOpenPrice() && Low[0] > aPivot[i]) {np = i; break;} 
			   i--;
			   }			   
			   BuyStop = aPivot[np - (StopLossLevel - 1)]; 
			   if(OrderOpenPrice() - BuyStop < MinStopLoss*_Point) BuyStop = OrderOpenPrice() - MinStopLoss*_Point;
			   if(DebugMode > 0) Print("BuyStop=",BuyStop);
			   if(NormalizeDouble(OrderOpenPrice(),Digits) > BuyStop) 
            {   
			      for(int k = 0 ; k < TriesNum; k++)
               {
               result = OrderModify(OrderTicket(),OrderOpenPrice(),
			                           BuyStop,
			                           OrderTakeProfit(),0,Lime);
               error=GetLastError();
                  if(error==0) break;
                  else {Sleep(5000); RefreshRates(); continue;}
               }   		 
            }
         }   
// - SELL Orders          
         if (mode==OP_SELL)
         {
            i = 0;
			   while(i <= 20)
			   { 
			   if (aPivot[i] > OrderOpenPrice() && High[0] < aPivot[i]) {np = i; break;}  
			   i++;
			   }		 
            SellStop = aPivot[np + (StopLossLevel - 1)];
            if(SellStop - OrderOpenPrice() < MinStopLoss*_Point) SellStop = OrderOpenPrice() + MinStopLoss*_Point;
            if(DebugMode > 0) Print("SellStop=",SellStop);
            if(NormalizeDouble(OrderOpenPrice(),Digits) < SellStop) 
            {
               for( k = 0 ; k < TriesNum; k++)
               {
               result = OrderModify(OrderTicket(),OrderOpenPrice(),
			                           SellStop,
			                           OrderTakeProfit(),0,Orange);
               error=GetLastError();
                  if(error==0) break;
                  else {Sleep(5000); RefreshRates(); continue;}
               }   
   			}	    
         }
      }
   }     
}   

void FindTargets()
{
   int trades = ScanTrades(1), np, prevTicket;
   double price;
   bool cond;  
   ArrayResize(Target,trades);
   
   int j=0;
   for (int cnt=0;cnt<OrdersTotal();cnt++)
   { 
   OrderSelect(cnt, SELECT_BY_POS);   
   int mode=OrderType();    
      if (OrderSymbol() == Symbol() && OrderMagicNumber() == Magic) 
      {
         if (mode==OP_BUY) 
         {
			price = OrderOpenPrice();
		
			   int i= 0;
			   while(i <= 20)
			   { 
			   if (aPivot[i] > price) {np = i; break;} 
			   i++;
			   }			      
            
            if(aPivot[np]   - OrderOpenPrice() < MinProfit * _Point) Target[j,4] = OrderOpenPrice() + MinProfit * _Point; 
            else Target[j,4] = aPivot[np];
            if(aPivot[np+1] - OrderOpenPrice() < MinProfit * _Point) Target[j,5] = OrderOpenPrice() + MinProfit * _Point; 
            else Target[j,5] = aPivot[np+1]; 
            if(aPivot[np+2] - OrderOpenPrice() < MinProfit * _Point) Target[j,6] = OrderOpenPrice() + MinProfit * _Point; 
            else Target[j,6] = aPivot[np+2];  
             
            if(DebugMode > 0) Print("j=",j," BUY tg1=",Target[j,4]," tg2=",Target[j,5]," tg3=",Target[j,6]);
            
            if(IsTesting() || IsVisualMode() || IsOptimization()) cond = StringSubstr(OrderComment(),0,12)!="split from #";
            else cond = StringSubstr(OrderComment(),0,6)!="from #";
                                   
            if(cond)
            {       
            Target[j,0] = 0;
            Target[j,1] = OrderTicket();
            Target[j,2] = 0;//OrderTicket();
            Target[j,3] = 0;//OrderTicket();
            if(TargetPct1 > 0) Target[j,7] = NormalizeDouble(0.01 * TargetPct1 * OrderLots()/lot_step,0)*lot_step; else Target[j,7] = 0;
            if(TargetPct2 > 0) Target[j,8] = NormalizeDouble(0.01 * TargetPct2 * OrderLots()/lot_step,0)*lot_step; else Target[j,8] = 0;
            if(TargetPct3 > 0) Target[j,9] = NormalizeDouble(0.01 * TargetPct3 * OrderLots()/lot_step,0)*lot_step; else Target[j,9] = 0;
            }
            else
            {
            if(IsTesting() || IsVisualMode() || IsOptimization()) prevTicket = StrToInteger(StringSubstr(OrderComment(),12));
            else prevTicket = StrToInteger(StringSubstr(OrderComment(),6));
            //Print("BUY: com=",OrderComment()," pT=",prevTicket);   
               if(prevTicket == Target[j,1] && prevTicket > 0) 
               {
               Target[j,0] = 1;
               Target[j,2] = OrderTicket();
               }
               else
               if(prevTicket == Target[j,2] && prevTicket > 0) 
               {
               Target[j,0] = 2;
               Target[j,3] = OrderTicket();
               }         
            }
            //Target[j,1] = OrderTicket();
            
            if(DebugMode > 0) Print("j=",j," ticket=",Target[j,1],",",Target[j,2],",",Target[j,3]," BUY lo1=",Target[j,7]," lo2=",Target[j,8]," lo3=",Target[j,9]);
         j++;
         //continue;
         }

         if (mode==OP_SELL) 
         {
			price = OrderOpenPrice();
			
			   i = 20;
			   while(i >= 0)
			   { 
			   if (aPivot[i] < price) {np = i; break;} 
			   i--;
			   }			      
            
            if(OrderOpenPrice() - aPivot[np]   < MinProfit * _Point) Target[j,4] = OrderOpenPrice() - MinProfit * _Point; 
            else Target[j,4] = aPivot[np];
            if(OrderOpenPrice() - aPivot[np-1] < MinProfit * _Point) Target[j,5] = OrderOpenPrice() - MinProfit * _Point; 
            else Target[j,5] = aPivot[np-1]; 
            if(OrderOpenPrice() - aPivot[np-2] < MinProfit * _Point) Target[j,6] = OrderOpenPrice() - MinProfit * _Point; 
            else Target[j,6] = aPivot[np-2];  
            
            if(DebugMode > 0) Print("j=",j," SELL tg1=",Target[j,4]," tg2=",Target[j,5]," tg3=",Target[j,6]);
            
            if(IsTesting() || IsVisualMode() || IsOptimization()) cond = StringSubstr(OrderComment(),0,12)!="split from #";
            else cond = StringSubstr(OrderComment(),0,6)!="from #";
                        
            if(cond)
            {       
            //if(OrderComment()!="") Print("SELL: com=",OrderComment());
            Target[j,0] = 0;
            Target[j,1] = OrderTicket();
            Target[j,2] = 0;//OrderTicket();
            Target[j,3] = 0;//OrderTicket();
            if(TargetPct1 > 0) Target[j,7] = NormalizeDouble(0.01 * TargetPct1 * OrderLots()/lot_step,0)*lot_step; else Target[j,7] = 0;
            if(TargetPct2 > 0) Target[j,8] = NormalizeDouble(0.01 * TargetPct2 * OrderLots()/lot_step,0)*lot_step; else Target[j,8] = 0;
            if(TargetPct3 > 0) Target[j,9] = NormalizeDouble(0.01 * TargetPct3 * OrderLots()/lot_step,0)*lot_step; else Target[j,9] = 0;
            }
            else
            {
            if(IsTesting() || IsVisualMode() || IsOptimization()) prevTicket = StrToInteger(StringSubstr(OrderComment(),12));
            else prevTicket = StrToInteger(StringSubstr(OrderComment(),6));
            //Print("SELL: com=",OrderComment()," pT=",prevTicket);   
               if(prevTicket == Target[j,1] && prevTicket > 0) 
               {
               Target[j,0] = 1;
               Target[j,2] = OrderTicket();
               }
               else
               if(prevTicket == Target[j,2] && prevTicket > 0) 
               {
               Target[j,0] = 2;
               Target[j,3] = OrderTicket();
               }         
            }
                        
            if(DebugMode > 0) Print("j=",j," tickets=",Target[j,1],",",Target[j,2],",",Target[j,3]," SELL lo1=",Target[j,7]," lo2=",Target[j,8]," lo3=",Target[j,9]);
         
         j++;
         }
      }
   }                           
}

void PartClose()   
{
   double Profit;
   int trades = ScanTrades(1);
        
   for (int i=0; i<trades; i++)  
   {
      for (int cnt=0;cnt<OrdersTotal();cnt++)
      { 
      if(!OrderSelect(cnt, SELECT_BY_POS)) continue;   
          
      if(OrderSymbol() != Symbol() || OrderMagicNumber() != Magic) continue; 
         if (OrderType()==OP_BUY)
         {
         Profit = (MarketInfo(Symbol(),MODE_BID) - OrderOpenPrice())/_Point;
            if(Profit > 0)
            {
               if(Target[i,0] == 0 && OrderTicket() == Target[i,1] && High[0] >= Target[i,4] && Target[i,7] > 0)
               {
               CloseAtMarket(OrderTicket(),Target[i,7],Aqua);
               
               }
      
               if(Target[i,0] == 1 && OrderTicket() == Target[i,2] && High[0] >= Target[i,5] && Target[i,8] > 0)
               {
               CloseAtMarket(OrderTicket(),Target[i,8],Aqua);
               
               }
      
               if(Target[i,0] == 2 && OrderTicket() == Target[i,3] && High[0] >= Target[i,6] && Target[i,9] > 0)
               {
               CloseAtMarket(OrderTicket(),Target[i,9],Aqua);
               
               }
            }       
         }
         else  
         if (OrderType()==OP_SELL)
         { 
         Profit = (OrderOpenPrice() - MarketInfo(Symbol(),MODE_ASK))/_Point;
            if(Profit > 0)
            {
               if(Target[i,0] == 0 && OrderTicket() == Target[i,1] && Low[0] <= Target[i,4] && Target[i,7] > 0)
               {
               CloseAtMarket(OrderTicket(),Target[i,7],Magenta);
               
               }
      
               if(Target[i,0] == 1 && OrderTicket() == Target[i,2] && Low[0] <= Target[i,5] && Target[i,8] > 0)
               {
               CloseAtMarket(OrderTicket(),Target[i,8],Magenta);
               
               }
      
               if(Target[i,0] == 2 && OrderTicket() == Target[i,3] && Low[0] <= Target[i,6] && Target[i,9] > 0)
               {
               CloseAtMarket(OrderTicket(),Target[i,9],Magenta);
               
               }
            }
         }
      }     
   }
}


bool CloseAtMarket(int ticket,double lot,color clr) 
{
   bool result = false; 
   int  ntr;
      
   int tries=0;
   while (!result && tries < TriesNum) 
   {
      RefreshRates();
      result=OrderClose(ticket,lot,OrderClosePrice(),Slippage,clr);
      tries++;
   }
   if (!result) Print("Error closing order : ",ErrorDescription(GetLastError()));
   return(result);
}





//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
//---- 
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
{
   if(iBars(Symbol(),0) < 100 || IsTradeAllowed()==false) return; 
      
   
   datetime cTime = iTime(Symbol(),0,0);
   datetime cDay  = iTime(Symbol(),PERIOD_D1,0);
          
   
   if(cDay > pDay && TimeCurrent() > cDay + DailyPivotHour*3600)
   { 
      for(int i=-10; i<=10; i++) aPivot[i+10] = Pivot(i);   
   pDay = cDay;
   }
   
   ChartComment();
   
   int Trades = ScanTrades(0);
      
   if (Trades > 0)
   {
      if(StopLossLevel > 0) InitialStop();
                 
      if(TrailStopMode>0)
      {
         if(cTime != pTime || fTime)
         {
         TrailStop();
         pTime = cTime;
         fTime = false;
         }
      }
      
      if(TargetPct1 > 0 || TargetPct2 > 0 || TargetPct3 > 0)
      {
      FindTargets();
      PartClose();
      }
   
      
   }      
      
//----
   return(0);
} //start()
//+------------------------------------------------------------------+